package org.springblade.deployment.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.deployment.entity.UpgradeProject;
import org.springblade.deployment.service.UpgradeProjectService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;


@RestController
@RequestMapping("upgrade/project")
@Api(value = "升级项目", tags = "升级项目相关接口")
public class UpgradeProjectController extends BladeController {

    @Autowired
    private UpgradeProjectService upgradeProjectService;

    @PostMapping("/upload-file/deployment")
    @ApiOperation("上传文件,部署项目")
    public R<String> upgradeProject(@RequestParam("file") @ApiParam(value = "二进制文件流") MultipartFile file,
                                     @ApiParam(value = "文件类型（0：前端，1：后端）") Integer projectType,
                                     @ApiParam("后端项目模块（gmetry-xxljob/gmetry-bi/gmetry-system/gmetry-lowcode/gmetry-pdf）")String projectModel,
                                     @ApiParam("env文件需要修改的模块名称（bi_version/pdf_version/xxljob_version/system_version/lowcode_version）")String envName,
                                     @ApiParam("项目版本号，修改.env文件需要用到")String projectVersion,
                                     @ApiParam(value = "项目升级日志",required = true)String upgradeLog){

        if (file.isEmpty()) {
            return R.fail("文件不能为空");
        }
        if (projectType == null){
            return R.fail("项目类型不能为空");
        }
        if (StrUtil.isBlank(upgradeLog)){
            return R.fail("升级日志不能为空");
        }
        UpgradeProject upgradeProject = new UpgradeProject();
        upgradeProject.setProjectType(projectType);
        upgradeProject.setUpgradeLog(upgradeLog);
        upgradeProject.setOperationUserId(getUser().getUserId());
        upgradeProject.setOperationUserName(getUser().getNickName());
        upgradeProject.setProjectVersion(projectVersion);
        // 后端项目升级需要的信息
        if (projectType == 1) {
            upgradeProject.setEnvName(envName);
            upgradeProject.setProjectModel(projectModel);
        }

        return R.data(upgradeProjectService.upgradeProject(file,upgradeProject));
    }

    @GetMapping("/list")
    @ApiOperation("查询部署记录")
    public R<IPage<UpgradeProject>> getUpgradeLog(UpgradeProject upgradeProject, Query query){
        upgradeProject.setOperationUserId(getUser().getUserId());
        LambdaQueryWrapper<UpgradeProject> queryWrapper = Condition.getQueryWrapper(upgradeProject).lambda().orderByDesc(UpgradeProject::getCreateTime);

        return R.data(upgradeProjectService.page(Condition.getPage(query), queryWrapper));
    }


    @GetMapping("/history/list")
    @ApiOperation("查询历史记录")
    public R<List<String>> getHistory(String model){

        return R.data(upgradeProjectService.history(model));
    }

    @GetMapping("/rollback")
    @ApiOperation("回退历史版本")
    public R<String> rollbackProject(@ApiParam(value = "回退的模块") @RequestParam String model,
                                     @ApiParam(value = "回退的版本号") @RequestParam String version,
                                     @ApiParam(value = ".env文件需要修应该的模块") @RequestParam String envModel){
        return R.data(upgradeProjectService.rollbackProject(model,version,envModel,getUser()));
    }
}
