package org.springblade.deployment.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.xmlbeans.impl.xb.xsdschema.Public;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.deployment.dto.TableInfoDto;
import org.springblade.deployment.entity.SqlBackupRecord;
import org.springblade.deployment.entity.UpgradeProjectSql;
import org.springblade.deployment.service.SqlBackupRecordService;
import org.springblade.deployment.service.UpgradeProjectSqlService;
import org.springblade.deployment.utils.UnzipUtils;
import org.springblade.deployment.vo.SqlBackupRecordVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

@RestController
@RequestMapping("upgrade/sql")
@Api(value = "升级项目", tags = "修改sql相关接口")
public class UpgradeProjectSqlController extends BladeController {

    @Autowired
    private UpgradeProjectSqlService upgradeProjectSqlService;

    @Autowired
    private SqlBackupRecordService sqlBackupRecordService;

    @PostMapping("/upload/exchange-sql")
    @ApiOperation("修改指定数据库的sql")
    public R<String> upgradeSql(@RequestParam("file") @ApiParam(value = "二进制文件流",required = true) MultipartFile file,
                                @RequestParam(value = "upgradeLog",required = false) @ApiParam(value = "项目升级日志")String upgradeLog){

        if (file.isEmpty()) {
            return R.fail("文件不能为空");
        }

        UpgradeProjectSql upgradeProjectSql = new UpgradeProjectSql();
        upgradeProjectSql.setUpgradeLog(StrUtil.isNotBlank(upgradeLog) ? upgradeLog : "");
        upgradeProjectSql.setOperationUserId(getUser().getUserId());
        upgradeProjectSql.setOperationUserName(getUser().getNickName());
        // 后端项目升级需要的信息

        return R.data(upgradeProjectSqlService.upgradeProjectSql(file,upgradeProjectSql));
    }

    @GetMapping("/get/logs")
    @ApiOperation("查询sql修改日志")
    public R<IPage<UpgradeProjectSql>> getSqlLogs(UpgradeProjectSql upgradeProjectSql, Query query){
        upgradeProjectSql.setOperationUserId(getUser().getUserId());
        LambdaQueryWrapper<UpgradeProjectSql> queryWrapper = Condition.getQueryWrapper(upgradeProjectSql).lambda().orderByDesc(UpgradeProjectSql::getCreateTime);

        return R.data(upgradeProjectSqlService.page(Condition.getPage(query), queryWrapper));
    }

    @GetMapping("/download/sql")
    @ApiOperation("下载sql")
    public void downloadSql(@RequestParam List<Integer> ids,HttpServletResponse response) {

        if (CollectionUtil.isEmpty(ids)) {
            throw new ServiceException("id不能为空");
        }
        String zipFileName = "digital_operation_" + LocalDateTime.now().format(DateTimeFormatter.ISO_DATE);
        String unzipPath= "";
        List<File> srcfile = new ArrayList<>();
        List<SqlBackupRecord> backupRecords = sqlBackupRecordService.listByIds(ids);
        for (SqlBackupRecord backupRecord : backupRecords) {
            if (backupRecord.getFileName().endsWith(".sql")) {
                File file = new File(backupRecord.getFilePath() + "/" + backupRecord.getFileName());
                if (file.exists()) {
                    srcfile.add(file);
                }
            } else {
                unzipPath = backupRecord.getFilePath() + "/" + backupRecord.getFileName().replace(".zip","");
                List<File> files = unzipFiles(backupRecord);
                srcfile.addAll(files);
            }
        }
        byte[] buf = new byte[1024];
        // 获取输出流
        BufferedOutputStream bos = null;
        try {
            bos = new BufferedOutputStream(response.getOutputStream());
        } catch (IOException e) {
            e.printStackTrace();
        }
        try {
            response.reset(); // 重点突出
            // 不同类型的文件对应不同的MIME类型
            response.setContentType("application/x-msdownload");
            response.setCharacterEncoding("utf-8");
            response.setHeader("Content-disposition", "attachment;filename=" + zipFileName + ".zip");

            // ZipOutputStream类：完成文件或文件夹的压缩
            ZipOutputStream out = new ZipOutputStream(bos);
            for (int i = 0; i < srcfile.size(); i++) {
                FileInputStream in = new FileInputStream(srcfile.get(i));
                // 给列表中的文件单独命名
                out.putNextEntry(new ZipEntry(srcfile.get(i).getName()));
                int len;
                while ((len = in.read(buf)) > 0) {
                    out.write(buf, 0, len);
                }
                out.closeEntry();
                in.close();
            }
            out.close();
            bos.close();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (StrUtil.isNotEmpty(unzipPath)) {
                FileUtil.del(unzipPath);
            }
        }
    }

    /**
     * 解压缩.zip文件
     */
    private List<File> unzipFiles(SqlBackupRecord backupRecord) {
        List<File> files = new ArrayList<>();
        String filePath = backupRecord.getFilePath() + "/" + backupRecord.getFileName();
        String unzipPath = filePath.replace(".zip","");
        // 对sql文件进行解压
        try {
            UnzipUtils.unzipFile(filePath ,unzipPath + "/");
            // 获取指定目录
            Path dir = Paths.get(unzipPath);

            List<Path> paths = Files.list(dir).collect(Collectors.toList());
            for (Path path : paths) {
                files.add(path.toFile());
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        return files;
    }

    @GetMapping("/backup/sql")
    @ApiOperation(value = "sql备份")
    public R<String> backupSql(@ApiParam("备份的表名，多个表用,隔开") @RequestParam(required = false) String tableNames,
                               @ApiParam("1:合并导出，2：分表导出") @RequestParam(required = false) Integer backupType,
                               @ApiParam("备注") @RequestParam(required = false) String remark,
                               @ApiParam("备份的Sql名称") @RequestParam(required = false) String fileName) {

        sqlBackupRecordService.backupSql(tableNames,backupType,remark,fileName);
        return R.data("备份成功");
    }

    @GetMapping("/table/list")
    @ApiOperation("查询表信息列表")
    public R<List<TableInfoDto>> getTableList(@RequestParam(required = false) String tableName) {

        return R.data(sqlBackupRecordService.getTableList(tableName));
    }

    @GetMapping("/sql/list")
    @ApiOperation("查询备份sql列表")
    public R<IPage<SqlBackupRecord>> getSqlBackupList(SqlBackupRecordVo backupRecord, Query query) {

        SqlBackupRecord record = BeanUtil.copyProperties(backupRecord, SqlBackupRecord.class);
        LambdaQueryWrapper<SqlBackupRecord> queryWrapper = Condition.getQueryWrapper(record).lambda().orderByDesc(SqlBackupRecord::getBackupTime);
        if (StrUtil.isNotEmpty(backupRecord.getBackupTimeStart()) && StrUtil.isNotEmpty(backupRecord.getBackupTimeEnd())) {
            queryWrapper.between(SqlBackupRecord::getBackupTime,backupRecord.getBackupTimeStart(),backupRecord.getBackupTimeEnd());
        }
        if (StrUtil.isNotEmpty(backupRecord.getRestoreTimeStart()) && StrUtil.isNotEmpty(backupRecord.getRestoreTimeEnd())) {
            queryWrapper.between(SqlBackupRecord::getRestoreTime,backupRecord.getRestoreTimeStart(),backupRecord.getRestoreTimeEnd());
        }

        return R.data(sqlBackupRecordService.page(Condition.getPage(query), queryWrapper));
    }

    @GetMapping("/restore/sql")
    @ApiOperation("恢复sql")
    public R<Boolean> restoreSql(@RequestParam List<Integer> ids) {
        if (CollectionUtil.isEmpty(ids)) {
            throw new ServiceException("id不能为空");
        }
        return R.data(sqlBackupRecordService.restore(ids));
    }
}
