package org.springblade.deployment.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.deployment.dto.TableInfoDto;
import org.springblade.deployment.entity.SqlBackupRecord;
import org.springblade.deployment.mapper.SqlBackupRecordMapper;
import org.springblade.deployment.service.SqlBackupRecordService;
import org.springblade.deployment.utils.ExecUtil;
import org.springblade.deployment.utils.UnzipUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * @author hepingguo
 * @version 1.0
 * @description: TODO
 * @date 2023/12/4 14:55
 */
@Service
public class SqlBackupRecordServiceImpl extends ServiceImpl<SqlBackupRecordMapper, SqlBackupRecord> implements SqlBackupRecordService {

    @Value("${upload.sql.folder}")
    private String sqlFolder;

    @Value("${upload.shell.folder}")
    private String shellFolder;

    @Value("${spring.datasource.dynamic.datasource.master.username}")
    private String databaseUser;

    @Value("${spring.datasource.dynamic.datasource.master.password}")
    private String databasePassword;

    @Value("${docker.container}")
    private String containerName;

    @Autowired
    private ExecUtil execUtil;

    @Override
    public void backupSql(String tableNames, Integer backupType,String remark,String fileName) {
        if (StrUtil.isNotEmpty(fileName)) {
            String name = fileName;
            if (backupType != null && backupType == 2) {
                if (!fileName.endsWith(".zip")) {
                    name = fileName + ".zip";
                }
            } else {
                if (!fileName.endsWith(".sql")) {
                    name = fileName + ".sql";
                }
            }
            int count = this.count(Wrappers.<SqlBackupRecord>lambdaQuery()
                    .eq(SqlBackupRecord::getFileName, name));
            if (count > 0) {
                throw new ServiceException("已存在同名称的文件");
            }
        }
        try {
            SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");
            SimpleDateFormat date = new SimpleDateFormat("yyyyMMdd");
            String sqlPath = sqlFolder + "/dump/" + date.format(new Date());
            String zipPath = "";
            File savePos = new File(sqlPath);
            if (!savePos.exists()) {  // 不存在，则创建该文件夹
                savePos.mkdirs();
            }
            String fileNames = null;
            // 如果表名为空则进行全量备份
            if (StrUtil.isEmpty(tableNames)) {

                String sqlName = "digital_operation" + format.format(new Date()) + ".sql";
                if (StrUtil.isNotEmpty(fileName)) {
                    if (fileName.endsWith(".sql")) {
                        sqlName = fileName;
                    } else {
                        sqlName = fileName + ".sql";
                    }
                }
                StringBuilder tableName = new StringBuilder();
                List<TableInfoDto> tableList = this.getTableList(null);
                for (TableInfoDto tableInfoDto : tableList) {
                    if (!tableInfoDto.getTableName().equals("sql_backup_record")
                            && !tableInfoDto.getTableName().equals("upgrade_project_sql")) {
                        tableName.append(tableInfoDto.getTableName()).append(" ");
                    }
                }
                // 从docker中导出sql文件到服务器指定目录
                execUtil.execShell("upgrade.sh",
                        // shell脚本在服务器上存放的位置,由于jar包无法获取到文件的路径只能先将资源文件下的shell脚本读取出来然后写会到服务器中
                        shellFolder,
                        // 项目类型
                        "5",
                        // 文件路径
                        sqlPath,
                        // docker的mysql容器名称
                        containerName,
                        // 用户名
                        databaseUser,
                        // 密码
                        databasePassword,
                        //表明
                        tableName.toString(),
                        // sql文件名称
                        sqlName);
                fileNames = sqlName;
            } else {
                List<String> tableList = new ArrayList<>();
                if (backupType != null && backupType == 2) {
                    for (String s : tableNames.split(",")) {
                        if (!s.equals("sql_backup_record")
                                && !s.equals("upgrade_project_sql")) {
                            tableList.add(s);
                        }
                    }

                } else {
                    tableList.add(tableNames.replace("sql_backup_record","").replace("upgrade_project_sql",""));
                }
                if (CollectionUtil.isEmpty(tableList)) {
                    return;
                }
                for (String table : tableList) {
                    String sqlName = "digital_operation_" + format.format(new Date()) + ".sql";
                    if (StrUtil.isNotEmpty(fileName)) {
                        if (fileName.endsWith(".sql")) {
                            sqlName = fileName;
                        } else {
                            sqlName = fileName + ".sql";
                        }
                    }
                    if (tableList.size() > 1) {
                        sqlName = table + "_" + format.format(new Date()) + ".sql";
                        zipPath = sqlPath + "/zip";
                        File file = new File(zipPath);
                        if (!file.exists()) {
                            file.mkdirs();
                        }
                        // 从docker中导出sql文件到服务器指定目录
                        execUtil.execShell("upgrade.sh",
                                // shell脚本在服务器上存放的位置,由于jar包无法获取到文件的路径只能先将资源文件下的shell脚本读取出来然后写会到服务器中
                                shellFolder,
                                // 项目类型
                                "5",
                                // 文件路径
                                zipPath,
                                // docker的mysql容器名称
                                containerName,
                                // 用户名
                                databaseUser,
                                // 密码
                                databasePassword,
                                //表名
                                table.replace(",", " "),
                                // sql文件名称
                                sqlName);
                    } else {
                        // 从docker中导出sql文件到服务器指定目录
                        execUtil.execShell("upgrade.sh",
                                // shell脚本在服务器上存放的位置,由于jar包无法获取到文件的路径只能先将资源文件下的shell脚本读取出来然后写会到服务器中
                                shellFolder,
                                // 项目类型
                                "5",
                                // 文件路径
                                sqlPath,
                                // docker的mysql容器名称
                                containerName,
                                // 用户名
                                databaseUser,
                                // 密码
                                databasePassword,
                                //表名
                                table.replace(",", " "),
                                // sql文件名称
                                sqlName);
                    }

                    fileNames = sqlName;
                }
                if (StrUtil.isNotEmpty(zipPath)) {
                    // 获取指定目录
                    Path dir = Paths.get(zipPath);
                    // 获取指定目录下的所有文件
                    List<Path> files = Files.list(dir).collect(Collectors.toList());
                    zipFiles(files, sqlPath, StrUtil.isNotEmpty(fileName) ? fileName : "digital_operation_" + format.format(new Date()));
                    FileUtil.del(zipPath);
                }
            }
            List<SqlBackupRecord> backupRecords = new ArrayList<>();
            if (StrUtil.isNotEmpty(fileNames)) {
                String filePath = sqlPath + "/" + fileNames;
                File file = new File(filePath);
                if (file.exists()) {
                    SqlBackupRecord backupRecord = new SqlBackupRecord();
                    backupRecord.setFileName(fileNames);
                    backupRecord.setFilePath(sqlPath);
                    backupRecord.setFileSize(file.length());
                    backupRecord.setBackupTime(LocalDateTime.now());
                    backupRecord.setRemarks(StrUtil.isNotBlank(remark) ? remark : "手动备份");
                    backupRecords.add(backupRecord);
                } else {
                    String zipName = StrUtil.isNotEmpty(fileName) ? fileName : "digital_operation_" + format.format(new Date());
                    filePath = sqlPath + "/" + (zipName.endsWith(".zip") ? zipName : zipName + ".zip");
                    file = new File(filePath);
                    if (file.exists()) {
                        SqlBackupRecord backupRecord = new SqlBackupRecord();
                        backupRecord.setFileName(zipName.endsWith(".zip") ? zipName : zipName + ".zip");
                        backupRecord.setFilePath(sqlPath);
                        backupRecord.setFileSize(file.length());
                        backupRecord.setBackupTime(LocalDateTime.now());
                        backupRecord.setRemarks(StrUtil.isNotBlank(remark) ? remark : "手动备份");
                        backupRecords.add(backupRecord);
                    }
                }
            }
            if (CollectionUtil.isNotEmpty(backupRecords)) {
                this.saveBatch(backupRecords);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 对多文件进行压缩
     */
    private void zipFiles(List<Path> srcFiles, String sqlPath, String sqlName) throws IOException {

        String fileName = sqlPath + "/" + sqlName.replace(".sql",".zip");
        if (!fileName.endsWith(".zip")) {
            fileName = fileName + ".zip";
        }
        FileOutputStream fos = new FileOutputStream(fileName);
        ZipOutputStream zipOut = new ZipOutputStream(fos);
        //向压缩包中添加多个文件
        for (Path srcFile : srcFiles) {
            File fileToZip = srcFile.toFile();
            FileInputStream fis = new FileInputStream(fileToZip);
            ZipEntry zipEntry = new ZipEntry(fileToZip.getName());
            zipOut.putNextEntry(zipEntry);

            byte[] bytes = new byte[1024];
            int length;
            while((length = fis.read(bytes)) >= 0) {
                zipOut.write(bytes, 0, length);
            }
            fis.close();
        }
        zipOut.close();
        fos.close();
    }

    @Override
    public List<TableInfoDto> getTableList(String tableName) {

        return this.baseMapper.getTables(tableName);
    }

    @Override
    public Boolean restore(List<Integer> ids) {

        try {
            List<SqlBackupRecord> records = this.listByIds(ids);
            List<SqlBackupRecord> recordList = new ArrayList<>();
            for (SqlBackupRecord record : records) {
                if (record.getFileName().endsWith(".sql")) {
                    execUtil.execShell("upgrade.sh",
                            // shell脚本在服务器上存放的位置,由于jar包无法获取到文件的路径只能先将资源文件下的shell脚本读取出来然后写会到服务器中
                            shellFolder,
                            // 项目类型
                            "2",
                            // 文件上传目录
                            record.getFilePath(),
                            // 用户名
                            databaseUser,
                            // 密码
                            databasePassword,
                            //sql文件名称
                            record.getFileName());
                    record.setRestoreStatus(1);
                    record.setRestoreTime(LocalDateTime.now());
                    recordList.add(record);
                } else if (record.getFileName().endsWith(".zip")) {
                    String unzipPath = record.getFilePath() + "/unzip";
                    // 对sql文件进行解压
                    UnzipUtils.unzipFile(record.getFilePath() + "/" + record.getFileName() ,unzipPath + "/");
                    // 获取指定目录
                    Path dir = Paths.get(unzipPath);
                    // 获取指定目录下的所有文件
                    List<Path> files = Files.list(dir).collect(Collectors.toList());
                    files.forEach(paths -> execUtil.execShell("upgrade.sh",
                            // shell脚本在服务器上存放的位置,由于jar包无法获取到文件的路径只能先将资源文件下的shell脚本读取出来然后写会到服务器中
                            shellFolder,
                            // 项目类型
                            "2",
                            // 文件上传目录
                            unzipPath,
                            // 用户名
                            databaseUser,
                            // 密码
                            databasePassword,
                            //sql文件名称
                            paths.getFileName().toString()));
                    FileUtil.del(unzipPath);
                    record.setRestoreStatus(1);
                    record.setRestoreTime(LocalDateTime.now());
                    recordList.add(record);
                }
            }
            if (CollectionUtil.isNotEmpty(recordList)) {
                this.updateBatchById(recordList);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return true;
    }
}
