package org.springblade.deployment.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springblade.core.secure.BladeUser;
import org.springblade.deployment.entity.UpgradeProject;
import org.springblade.deployment.mapper.UpgradeProjectMapper;
import org.springblade.deployment.service.UpgradeProjectService;
import org.springblade.deployment.utils.ExecUtil;
import org.springblade.deployment.utils.FileUtils;
import org.springblade.deployment.utils.UnzipUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.xml.crypto.Data;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Stream;

@Service
public class UpgradeProjectServiceImpl extends ServiceImpl<UpgradeProjectMapper, UpgradeProject> implements UpgradeProjectService {

    @Value("${upload.api.folder}")
    private String apiFolder;

    @Value("${upload.api.unzip}")
    private String apiUnzip;

    @Value("${upload.webapp.folder}")
    private String webAppFolder;

    @Value("${upload.shell.folder}")
    private String shellFolder;

    @Autowired
    private ExecUtil execUtil;

    @Override
    public String upgradeProject(MultipartFile file, UpgradeProject upgradeProject) {
        // 后端路径
        String msg = "";
        String FOLDER_PATH = webAppFolder;
        if (upgradeProject.getProjectType() == 0){
            FOLDER_PATH = apiFolder;
        }
        try {
            // 上传文件到指定目录
            File savePos = new File(FOLDER_PATH);
            if (!savePos.exists()) {  // 不存在，则创建该文件夹
                savePos.mkdir();
            }
            // 上传该文件至该文件夹下
            File newFile = new File(FOLDER_PATH + "/" + file.getOriginalFilename());
            if (newFile.exists()){
                // 如果存在则先删除
                FileUtil.del(FOLDER_PATH + "/" + file.getOriginalFilename());
            }
            file.transferTo(newFile);
            // 对前端文件进行解压
            if (upgradeProject.getProjectType() == 0) {
//                UnzipUtils.unzipFile(FOLDER_PATH + "/" + file.getOriginalFilename(), apiUnzip);
                FileUtils.unpack(newFile,new File(apiUnzip));
            }
            Map<String, String> deploymentMap = execUtil.execShell("upgrade.sh",
                    // shell脚本在服务器上存放的位置,由于jar包无法获取到文件的路径只能先将资源文件下的shell脚本读取出来然后写会到服务器中
                    shellFolder,
                    // 项目类型
                    upgradeProject.getProjectType().toString(),
                    // 前端解压目录
                    apiUnzip,
                    // 后端镜像目录
                    webAppFolder,
                    // 脚本目录
                    shellFolder,
                    // 前端上传的文件
                    file.getOriginalFilename(),
                    // 项目升级的模块
                    StrUtil.isNotBlank(upgradeProject.getProjectModel()) ? upgradeProject.getProjectModel().trim() : "",
                    // .env中项目的版本名称
                    StrUtil.isNotBlank(upgradeProject.getEnvName()) ? upgradeProject.getEnvName() : "",
                    // 项目的版本号
                    StrUtil.isNotBlank(upgradeProject.getProjectVersion()) ? upgradeProject.getProjectVersion() : "");

            // 对升级的结果进行分析
            if (CollectionUtil.isNotEmpty(deploymentMap)){
                // 如果出现错误的信息那么一定是部署出错了
                if (StrUtil.isNotBlank(deploymentMap.get("error"))){
                    upgradeProject.setResultMsg(deploymentMap.get("error"));
                    upgradeProject.setUpgradeResult(1);
                    msg = deploymentMap.get("error");
                } else if (StrUtil.isNotBlank(deploymentMap.get("msg"))){
                    // 这里可能会有空的现象
                    upgradeProject.setResultMsg(deploymentMap.get("msg"));
                    upgradeProject.setUpgradeResult(1);
                    msg = deploymentMap.get("msg");
                }
            } else {
                upgradeProject.setResultMsg("未知错误");
                upgradeProject.setUpgradeResult(0);
                msg = "未知错误";
            }
            upgradeProject.setResultMsg(msg);
            // oldVersion: bi_version=1.1.0 只取等号后面的版本号
            if (upgradeProject.getProjectType() == 1) {
                String oldVersion = getProjectOldVersionFromEnv(shellFolder, upgradeProject.getEnvName());
                upgradeProject.setOldVersion(oldVersion.split("=")[1]);
            }
            // 将升级的信息进行落库
            this.save(upgradeProject);
        } catch (Exception e){
            e.printStackTrace();
        }
        // 升级完成后删除文件
        FileUtil.del(FOLDER_PATH + "/" + file.getOriginalFilename());
        return msg;
    }

    @Override
    public List<String> history(String model) {
        List<String> tags = new ArrayList<>();
        Map<String, String> deploymentMap = execUtil.execShell("upgrade.sh",
                // shell脚本在服务器上存放的位置,由于jar包无法获取到文件的路径只能先将资源文件下的shell脚本读取出来然后写会到服务器中
                shellFolder,
                // 项目类型
                "3",
                model);
        if (CollectionUtil.isNotEmpty(deploymentMap)){

            // 如果出现错误的信息那么一定是部署出错了
            if (StrUtil.isNotBlank(deploymentMap.get("error"))){
                return tags;
            } else if (StrUtil.isNotBlank(deploymentMap.get("msg"))){
                String msg = deploymentMap.get("msg");
                String[] split = msg.split(" ");
                for (int i = 1; i < split.length; i++) {
                    tags.add(split[i]);
                }
            }
        }

        return tags;
    }

    @Override
    public String rollbackProject(String model, String version, String envModel, BladeUser user) {

        UpgradeProject project = new UpgradeProject();
        project.setProjectModel(model);
        project.setProjectVersion(version);
        project.setUpgradeLog("回退历史版本");
        project.setEnvName(envModel);
        project.setProjectType(0);
        project.setOperationUserId(user.getUserId());
        project.setOperationUserName(user.getNickName());
        project.setCreateTime(LocalDateTime.now());
        String msg = "回退失败";
        Map<String, String> map = execUtil.execShell("upgrade.sh", shellFolder, "-1",shellFolder, model, envModel, version);
        if (CollectionUtil.isNotEmpty(map)) {
            if (StrUtil.isNotBlank(map.get("error"))) {
                project.setUpgradeResult(0);
                project.setResultMsg(map.get("error"));
                msg = "升级成功";
            } else if (StrUtil.isNotBlank(map.get("msg"))) {
                project.setUpgradeResult(0);
                project.setResultMsg(map.get("msg"));
                msg = "升级成功";
            }
        } else {
            project.setUpgradeResult(1);
            project.setResultMsg("未知错误");
        }
        this.save(project);
        return msg;
    }

    private String getProjectOldVersionFromEnv(String shellFolder,String envName) {
        AtomicReference<String> projectVersion = new AtomicReference<>("");
        // 读取文件内容到Stream流中，按行读取
        Stream<String> lines;
        try {
            lines = Files.lines(Paths.get(shellFolder+"/.env"));
            if (lines == null){
                return projectVersion.get();
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        // 随机行顺序进行数据处理
        lines.forEach(ele -> {
            if (ele.contains(envName)){
                projectVersion.set(ele);
            }
        });

        return projectVersion.get();
    }

}
