package org.springblade.deployment.service.impl;

import java.io.File;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import org.springblade.deployment.entity.SqlBackupRecord;
import org.springblade.deployment.entity.UpgradeProjectSql;
import org.springblade.deployment.mapper.UpgradeProjectSqlMapper;
import org.springblade.deployment.service.SqlBackupRecordService;
import org.springblade.deployment.service.UpgradeProjectSqlService;
import org.springblade.deployment.utils.ExecUtil;
import org.springblade.deployment.utils.UnzipUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.StrUtil;

@Service
public class UpgradeProjectSqlServiceImpl extends ServiceImpl<UpgradeProjectSqlMapper, UpgradeProjectSql> implements UpgradeProjectSqlService {

    private final String DECOMPRESSION_PASS = "Platomix@sql.tar";

    @Value("${upload.sql.folder}")
    private String sqlFolder;

    @Value("${upload.shell.folder}")
    private String shellFolder;

    @Value("${spring.datasource.dynamic.datasource.master.username}")
    private String databaseUser;

    @Value("${spring.datasource.dynamic.datasource.master.password}")
    private String databasePassword;

    @Autowired
    private ExecUtil execUtil;

    @Autowired
    private SqlBackupRecordService sqlBackupRecordService;



    @Override
    public String upgradeProjectSql(MultipartFile file, UpgradeProjectSql upgradeProjectSql) {
        String msg = "";
        SimpleDateFormat date = new SimpleDateFormat("yyyyMMdd");
        // 上传文件到指定目录
        String sqlPath = sqlFolder + "/upload/" + date.format(new Date());
        try {
            File savePos = new File(sqlPath);
            if (!savePos.exists()) {  // 不存在，则创建该文件夹
                savePos.mkdirs();
            }
            String filePath = sqlPath + "/" + file.getOriginalFilename();
            // 上传该文件至该文件夹下
            File newFile = new File(filePath);
            if (newFile.exists()){
                // 如果存在则先删除
                FileUtil.del(filePath);
            }
            file.transferTo(newFile);
            File files = new File(filePath);
            if (files.exists()) {
                SqlBackupRecord backupRecord = new SqlBackupRecord();
                backupRecord.setFileName(file.getOriginalFilename());
                backupRecord.setFilePath(sqlPath);
                backupRecord.setFileSize(files.length());
                backupRecord.setBackupTime(LocalDateTime.now());
                backupRecord.setDataSource(1);
                sqlBackupRecordService.save(backupRecord);
                msg = "上传成功";
            }
        } catch (Exception e){
            e.printStackTrace();
        }

        return msg;
    }

    @Override
    public File dumpSql(String fileName,String dockerContainer) {
        File sqlFile = null;
        try {
            String sqlName = fileName.replace(".sql", "") + ".sql";
            File savePos = new File(sqlFolder);
            if (!savePos.exists()) {  // 不存在，则创建该文件夹
                savePos.mkdir();
            }
            String filePath = sqlFolder + "/" + sqlName;
            // 从该文件夹下下载sql文件
            File newFile = new File(filePath);
            if (newFile.exists()){
                // 如果存在则先删除
                FileUtil.del(filePath);
            }
            // 从docker中导出sql文件到服务器指定目录
            Map<String, String> map = execUtil.execShell("upgrade.sh",
                    // shell脚本在服务器上存放的位置,由于jar包无法获取到文件的路径只能先将资源文件下的shell脚本读取出来然后写会到服务器中
                    shellFolder,
                    // 项目类型
                    "4",
                    // docker的mysql容器名称
                    dockerContainer,
                    // 用户名
                    databaseUser,
                    // 密码
                    databasePassword,
                    //sql文件名称
                    sqlName);
            sqlFile = new File(filePath);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return sqlFile;
    }
}
