package org.springblade.deployment.utils;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ArrayUtils;
import org.springframework.stereotype.Component;

import java.io.*;
import java.util.HashMap;
import java.util.Map;

/**
 * 执行工具类，可以执行shell脚本
 * @Author: liyongjie
 * @Description:
 * @Date: 2022/8/11 4:58 PM
 */
@Slf4j
@Component
public class ExecUtil {

    /**
     * 执行shell脚本,脚本放在项目的resource目录下
     * @param scriptName 脚本文件名，带不带sh后缀都可以
     * @param para 参数数组
     */
    public Map<String,String> execShell(String scriptName,String shellPath, String ... para) {
        Map<String,String> map = new HashMap<>();
        try {
            String scriptPath = readerShell(shellPath,scriptName);
            if (StrUtil.isBlank(scriptPath)){
                log.error("没有找到shell脚本...");
                return null;
            }
            ProcessBuilder pb = new ProcessBuilder("chmod", "+x", scriptPath);
            Process authCmd = pb.start();
            authCmd.waitFor();
            String[] cmd = new String[]{"./" + scriptName};
            //为了解决参数中包含空格
            cmd= ArrayUtils.addAll(cmd,para);
            // 添加操作系统
            cmd = ArrayUtils.add(cmd, getOS());
            pb = new ProcessBuilder(cmd);
            pb.directory(new File(scriptPath.substring(0, scriptPath.length() - scriptName.length() - 1)));
            Process ps = pb.start();
            ps.waitFor();

            BufferedReader stdInput = new BufferedReader(new InputStreamReader(ps.getInputStream()));
            BufferedReader stdError = new BufferedReader(new InputStreamReader(ps.getErrorStream()));
            String s;
            StringBuffer msg = new StringBuffer("");
            StringBuffer error = new StringBuffer("");
            while ((s = stdInput.readLine()) != null) {
                msg.append(s).append(" ");
                log.info("{} script 日志:{}", scriptName, s);
            }
            while ((s = stdError.readLine()) != null) {
                error.append(s).append(" ");
                log.error("{} script 异常:{}", scriptName, s);
            }
            if (StrUtil.isNotBlank(msg)){
                map.put("msg",msg.toString());
            } else if (StrUtil.isNotBlank(error)) {
                map.put("error",error.toString());
            }
            if (StrUtil.isBlank(msg) && StrUtil.isBlank(error)){
                map.put("msg","");
            }
        } catch (Exception e) {
            log.error("exec shell e", e);
        }

        return map;
    }

    private static String getOS() {
        String property = System.getProperty("os.name");
        if (property.startsWith("Mac")) {
            return "Mac";
        } else if (property.startsWith("Linux")) {
            return "Linux";
        } else if (property.startsWith("Windows")) {
            return "win";
        }
        return "";
    }

    private String readerShell(String shellPath,String scriptName){

        File file = new File(shellPath+"/"+scriptName);
        if (file.exists()){
            return shellPath+"/"+scriptName;
        }
        String scriptPath = "";
        //返回读取指定资源的输入流
        InputStream inputStream = this.getClass().getClassLoader().getResourceAsStream(scriptName);
        // 如果没有读取到资源则返回
        if (inputStream == null){
            return null;
        }
        try {
            copyInputStreamToFile(inputStream,file);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        scriptPath = file.getAbsoluteFile().toString();
        return scriptPath;
    }

    private static void copyInputStreamToFile(InputStream inputStream, File file)
            throws IOException {

        try (FileOutputStream outputStream = new FileOutputStream(file)) {

            int read;
            byte[] bytes = new byte[1024];

            while ((read = inputStream.read(bytes)) != -1) {
                outputStream.write(bytes, 0, read);
            }
        }

    }
}

